#! /usr/bin/perl -w

use strict;
use warnings;
use feature qw(say);

use Config::YAML;
use Data::Dumper;
use File::Slurp;
use Getopt::Long qw( :config posix_default bundling );
use Log::Dispatch;
use LWP::UserAgent;

my $config_file = '/etc/phishing_feeds.yml';

my $cache_dir = '/var/cache/phishing_feeds';
my $feeds_dir = '/var/lib/phishing_feeds';

my %args;

sub showhelp {
    my $msg= <<EOF;
usage: $0 [ OPTION ]

Download phishing feeds for use them with Mail::SpamAssassin::Plugin::Phishing

OPTIONS:
    -c --config                             Config file to use (Default: /etc/phishing_feeds.yml)
    -h --help                               Print this menu
    -D --debug                              Debug mode
    --stdout                                Log to stdout

Require config file (Default: /etc/phishing_feeds.yml) with the following content: 

---
feeds_url:
  openphish: https://openphish.com/feed.txt
  phishtank: http://data.phishtank.com/data/online-valid.csv.
# Optionnal (default values here)
#cache_dir: '/var/cache/phishing_feeds'
#feeds_dir: '/var/lib/phishing_feeds'

EOF
say( "$msg" );
}
GetOptions(
    "c|config=s"        => \$config_file,
    "h|help"            => \$args{help},
    "D|debug"           => \$args{debug},
    "stdout"            => \$args{stdout},
) or say STDERR "Error on command line arguments" && showhelp && exit 1;

# Print help and exit if -h / --help
showhelp && exit 0 if ( $args{help} );

my $log = Log::Dispatch->new();
require Log::Dispatch::Syslog;
require Log::Dispatch::Screen;
$log->add(Log::Dispatch::Syslog->new(
        min_level   => $args{debug} ? 'debug' : 'info',
        ident       => 'get_phishing_feeds.pl',
    )
);
$log->add(Log::Dispatch::Screen->new(
        name        => "STDERR",
        min_level   => 'error',
));

if ($args{stdout}) {
    $log->add(Log::Dispatch::Screen->new(
            name        => "Screen",
            min_level   => $args{debug} ? 'debug' : 'info',
    ));
}

# Load configuration file
my $config = Config::YAML->new(config => "$config_file");

$cache_dir = $config->{cache_dir} if $config->{cache_dir};
$feeds_dir = $config->{feeds_dir} if $config->{feeds_dir};

unless (keys %{$config->{feeds_url}}) {
    say STDERR "No feeds configured in $config_file";
    showhelp;
    exit 1;
}

# Check if cache dir is present and exists
unless (-d $cache_dir && -r $cache_dir) {
    $log->error("Cache directory $cache_dir doesn't exist or cannot be accesible for current user\n");
    $log->error("Create it with: mkdir -p $cache_dir\n");
    exit 1;
}

unless (-d $feeds_dir && -r $feeds_dir) {
    $log->error("Feeds directory $feeds_dir doesn't exist or cannot be accesible for current user\n");
    $log->error("Create it with: mkdir -p $feeds_dir\n");
    exit 1;
}

my $ua = LWP::UserAgent->new();

foreach my $feed (keys %{$config->{feeds_url}}) {
    my $feed_url = $config->{feeds_url}->{$feed};
    (my $ext = $feed_url) =~ s/^.+\.(csv|txt)\.?$/$1/;
    my $feed_name = $feed . '.' . $ext;
    $log->debug('Feed: ' . $feed . " / URL: " . $feed_url . " / FILE: " . $feed_name . "\n");

    my ($response, $last_modified, $last_modified_cached);

    if (-f $cache_dir . '/' . $feed) {
        $last_modified_cached = read_file($cache_dir . '/' . $feed);
    }

    if ($last_modified_cached) {
        # Check Last-Modified header on remote file
        $response = $ua->head($feed_url);
        unless ($response->is_success) {
            $log->error("Error get head on " . $feed_url. ". CODE: " . $response->code . ": " . $response->status_line . "\n");
            next;
        }
        $last_modified = $response->header("Last-Modified");

        if ($last_modified_cached eq $last_modified) {
            $log->info("No changes on feed " . $feed_url . " : SKIP\n");
            next;
        }
    }

    $log->info("Download feed $feed_url...\n");
    $response = $ua->get($feed_url, ':content_file' => $feeds_dir . '/' . $feed_name);
    unless ($response->is_success) {
        $log->error("Error get $feed_url FAILED. CODE: " . $response->code . ": " . $response->status_line . "\n");
        next;
    }
    # Write Response Last-Modified header in cache file
    if (open(my $fh, ">", $cache_dir . '/' . $feed)) {
        print $fh $response->header('Last-Modified');
        close $fh;
    } else {
        $log->error("Cannot open file " . $cache_dir . '/' . $feed . ": " . $! . "\n");
    }
}



